package clustering;
/*
 * This file is part of CANE - A Clustering Algorithm for Non-Euclidean spaces.
 *
 * Copyright (C) 2014 Benjamin Eltzner
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */

import java.util.List;

public interface Clusterable
{
    public long distanceSquare(Clusterable that);

    /**
     * Changes the Clusterable and returns it.
     */
    public Clusterable moveToCenter(List<Clusterable> cluster);

    /**
     * Split the center of a cluster in two points along the direction
     * of greatest variance.
     */
    public List<Clusterable> children(List<Clusterable> cluster);

    public double[] projection(List<Clusterable> direction, List<Clusterable> cluster)
           throws NotImplementedException;

    public String exportCenter();

    public int label() throws NotImplementedException;
    public int dimension();
    public Clusterable clone();
}
