package clustering;
/*
 * This file is part of CANE - A Clustering Algorithm for Non-Euclidean spaces.
 *
 * Copyright (C) 2014 Benjamin Eltzner
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */


import java.util.List;

import org.w3c.dom.Document;

public interface HierarchicalClusterer
{

    public abstract AbstractNode<?> root();

    /**
     * Returns a shallow copy of the leaves to protect
     * integrity of the tree.
     */
    public abstract <T extends AbstractNode<?>> List<T> leaves();

    public abstract Document toXML();

    public static abstract class AbstractNode<T extends AbstractNode<T>>
    {
        private T m_parent, m_child1, m_child2;

        protected void setParent(T parent)
        {
            if (m_parent == null)
            {
                m_parent = parent;
            }
            else
            {
                System.err.println("Tried to reset parent of node!");
            }
        }

        protected void setChild1(T child1)
        {
            if (m_child1 == null)
            {
                m_child1 = child1;
            }
            else
            {
                System.err.println("Tried to reset child1 of node!");
            }
        }

        protected void setChild2(T child2)
        {
            if (m_child2 == null)
            {
                m_child2 = child2;
            }
            else
            {
                System.err.println("Tried to reset child2 of node!");
            }
        }

        public T parent()
        {
            return m_parent;
        }

        public T child1()
        {
            return m_child1;
        }

        public T child2()
        {
            return m_child2;
        }

        public T sibling(T caller)
        {
            return (caller.equals(m_child2) ? m_child1 :
                   (caller.equals(m_child1) ? m_child2 : null));
        }

        protected void disappear (T caller)
        {
            T new_child = (caller.equals(m_child2) ? m_child1 :
                          (caller.equals(m_child1) ? m_child2 : null));
            if (new_child == null)
            {
                System.out.println("Invalid caller!");
                return;
            }
            if (m_parent != null)
            {
                m_parent.changeChild(this, new_child);
            }
            new_child.newParent();
        }

        protected void changeChild(AbstractNode<T> old_child, T new_child)
        {
            if (m_child1.equals(old_child))
            {
                m_child1 = new_child;
            }
            else if (m_child2.equals(old_child))
            {
                m_child2 = new_child;
            }
            else
            {
                System.err.println("Invalid child to replace!");
            }
        }

        protected void newChild(T old_child, T new_child)
        {
            if (m_child1.equals(old_child))
            {
                m_child1 = new_child;
            }
            else if (m_child2.equals(old_child))
            {
                m_child2 = new_child;
            }
            else
            {
                System.err.println("Invalid child to replace!");
            }
        }

        protected void newParent()
        {
            m_parent = m_parent.parent();
        }

        public void orphan()
        {
            m_parent = null;
        }
    };
}